﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using Telerik.Windows.DragDrop.Behaviors;
using SWC = System.Windows.Controls;

namespace Telerik.Windows.Controls.TreeView
{
	/// <summary>
	/// 
	/// </summary>
	public class TreeViewDragDropHelper : Telerik.Windows.DragDrop.Behaviors.DragDropHelper<TreeViewDragDropBehavior, Telerik.Windows.DragDrop.Behaviors.DragDropState>
	{
		/// <summary>
		/// Finds the RadTreeView or RadTreeViewItem that contains the provided element.
		/// </summary>
		/// <param name="element">The element that raised the DragInitialize event.</param>
		protected override FrameworkElement FindDragSource(FrameworkElement element)
		{
			SWC.ItemsControl target = element as RadTreeViewItem ?? element.ParentOfType<RadTreeViewItem>();
			if (target != null)
			{
				target = ((RadTreeViewItem)target).GetParent();
			}
			if (target == null)
			{
				target = element as RadTreeView ?? element.ParentOfType<RadTreeView>();
			}
			return target;
		}

		/// <summary>
		/// Finds the RadTreeViewItem that contains the provided element.
		/// </summary>
		/// <param name="element">The element that raised the DragInitialize event.</param>
		protected override FrameworkElement FindDraggedItem(FrameworkElement element)
		{
			return element as RadTreeViewItem ?? element.ParentOfType<RadTreeViewItem>();
		}

		/// <summary>
		/// Finds the RadTreeView or RadTreeViewItem that should accept the dragged items.
		/// </summary>
		/// <param name="element">The element that raised the Drop event.</param>
		/// <param name="data">The DataObject for the current drag operation.</param>
		protected override FrameworkElement FindDropItemsControl(FrameworkElement element, object data)
		{
			var item = element as RadTreeViewItem ?? element.ParentOfType<RadTreeViewItem>();
			while (item != null && (item.ItemsSource == null || !AreCompatible(this.GetItemType(item), data)))
			{
				item = item.GetParent() as RadTreeViewItem;
			}

			return (SWC.ItemsControl)item ?? element as RadTreeView ?? element.ParentOfType<RadTreeView>();
		}

		/// <summary>
		/// Finds a RadTreeView or RadTreeViewItem from the target control that contains the provided element.
		/// </summary>
		/// <param name="element">The element that raised the Drop event.</param>
		protected override FrameworkElement FindDropTarget(FrameworkElement element)
		{
			SWC.ItemsControl target = element as RadTreeViewItem ?? element.ParentOfType<RadTreeViewItem>();
			if (target == null)
			{
				target = element as RadTreeView ?? element.ParentOfType<RadTreeView>();
			}
			return target;
		}

		/// <summary>
		/// Returns the SelectedItems list of the RadTreeView that contains the dragged item and adds the dragged item to the list if it is not already present there.
		/// </summary>
		/// <param name="draggedItem">The source control of the drag operation.</param>
		/// <returns>The items that should be dragged.</returns>
		protected override IEnumerable<object> GetDraggedItems(FrameworkElement draggedItem)
		{
			var treeViewItem = (RadTreeViewItem)draggedItem;
			var treeView = treeViewItem.ParentTreeView;

			// Select the dragged item
			if (!treeView.SelectedItems.Contains(treeViewItem.DataContext))
			{
				treeView.SelectedItems.Add(treeViewItem.DataContext);
			}

			// Filter the selected items to contain only one type
			var selectedItems = GetItemsOfType(treeView.SelectedItems.OfType<object>().ToList(), GetInferredItemType(this.GetItemsSource(treeViewItem.GetParent())));

			treeView.SelectedItems.Clear();
			foreach (var item in selectedItems)
			{
				treeView.SelectedItems.Add(item);
			}

			return selectedItems.OfType<object>();
		}

		protected override void Drop(FrameworkElement dropTarget, FrameworkElement dropItemsControl, object data, Type itemType)
		{
			var targetTreeViewItem = dropItemsControl as RadTreeViewItem;
			if (targetTreeViewItem != null)
			{
				targetTreeViewItem.IsExpanded = true;
			}

			base.Drop(dropTarget, dropItemsControl, data, itemType);
		}

		//protected override DragDropEffects DragOver(FrameworkElement dropTarget, FrameworkElement dropItemsControl, object data, Type itemType)
		//{
		//    var targetTreeView = dropItemsControl as RadTreeView ?? dropItemsControl.ParentOfType<RadTreeView>();
		//    if (targetTreeView != null)
		//    {
		//        if (dropTarget != dropItemsControl)
		//        {
		//            var targetTreeViewItem = dropTarget as RadTreeViewItem;
		//            if (targetTreeViewItem != null)
		//            {
		//                targetTreeViewItem.DropPosition = DropPosition.Before;
		//                targetTreeView.ShowBetweenItemsDragCue(targetTreeViewItem);
		//            }
		//        }
		//        else
		//        {
		//            targetTreeView.HideBetweenItemsDragCue();
		//        }
		//    }

		//    return base.DragOver(dropTarget, dropItemsControl, data, itemType);
		//}

		private bool AreCompatible(Type type, object data)
		{
			var payloadTypes = DataObjectHelper.GetFormats(data, false).Select(format => Type.GetType(format)).OfType<Type>();

			var convertibleTypes = this.DataConverter.GetConvertToFormats().Select(format => Type.GetType(format)).OfType<Type>();

			return payloadTypes.Union(convertibleTypes).Any(t => t.IsAssignableFrom(type));
		}
	}
}